/* File: CSplitSys.cpp
 * Version 1.0 (AUG 25, 2010)
 * --------------------------------------------------------------------------
 * Objective: Define the member functions of CSplitSys
 *
 *	Written by Jialiang Yang
 * --------------------------------------------------------------------------
 */

// Body of quartet.h
#include <string>
#include <algorithm>
#include <iostream>
#include <fstream>
#include "CSplitSys.h"
#include "auxilary.h"
                                             
// Operator constructor
CSplitSys& CSplitSys::operator=(const CSplitSys& rhs)
{
	m_iNumber = rhs.m_iNumber;
	m_gName = rhs.m_gName;
	m_gTriplet = rhs.m_gTriplet;
	m_gQuartet = rhs.m_gQuartet;
	m_gQuintet = rhs.m_gQuintet;
	m_gSplit = rhs.m_gSplit;
	return *this;
}   

// Function to order quartets
bool CmpInd(const CQuartet &qA, const CQuartet &qB) 
{ 
     return qA.GetIndex() < qB.GetIndex();
}

// Reorder the quartets
void CSplitSys::OrderQuartets()
{
     sort(m_gQuartet.begin(), m_gQuartet.end(), CmpInd);
 }                                                                                                                                     
 
// Function to order triplets
bool CmpTriplet(const CTriplet &tA, const CTriplet &tB) 
{ 
     return tA.GetIndex() < tB.GetIndex();
}

// Reorder the triplets
void CSplitSys::OrderTriplets()
{
     sort(m_gTriplet.begin(), m_gTriplet.end(), CmpTriplet);
 }           
 
 
// Delete the split in split_sys that has weight 0
void CSplitSys::ShrinkSplits(vector<CSplit>& sTemp)
{
    vector<CSplit>::iterator iter;
    for(iter = sTemp.begin(); iter != sTemp.end(); ++iter)
	   {
	      if((*iter).GetWeight() == 0) 
		  {
              sTemp.erase(iter);
		      --iter;
		  }
	   } 
}

void CSplitSys::Output(string& sOutputFile)
{
	// Output the final full CSplits and weights into the file fullCSplit.txt.
     ofstream fout;
	 sOutputFile  = sOutputFile + ".nex";
	 
     fout.open( sOutputFile.c_str(),   ios_base::out   |   ios_base::trunc);  
		    
     if(!fout) 
     {   
        cerr << "Error opening " << sOutputFile << " for output!" << endl;  
        system("pause"); 
        exit(-1);  
     }
     else
     {
	     // Output taxa number and labels
	     fout << "#nexus" << endl;
		 fout << endl;
		 
		 fout << "BEGIN Taxa;" << endl;
		 fout << "DIMENSIONS ntax=" << m_iNumber << ";" << endl;
		 fout << "TAXLABELS" << endl;
		 
		 unsigned int ix=1;
		 vector<string>::iterator iter1;
		 for(iter1 = m_gName.begin(); iter1 != m_gName.end(); ++iter1)
         {
		     fout << "[" << ix << "] " << "'" << *iter1 << "'" << endl;
			 ++ix;
		 }
		 fout << ";" << endl;
		 fout << "END; [Taxa]" << endl;
		 
		 fout << endl;
		 
		 // Output CSplits and their weights
		 fout << "BEGIN splits;" << endl;
		 fout << "DIMENSIONS ntax=" << m_iNumber << " nsplits=" << m_gSplit.size() << ";" << endl;
		 fout << "FORMAT labels=no weights=yes confidences=no intervals=no;" << endl;
		 fout << "MATRIX" << endl;
		 
		 unsigned int ia=1;
         for(vector<CSplit>::iterator iter = m_gSplit.begin(); iter != m_gSplit.end(); ++iter)
         {
		    vector<unsigned int> gLeft, gRight;
            gLeft = (*iter).GetLblk();
			gRight = (*iter).GetRblk();
		    unsigned int iLlen = gLeft.size();
			unsigned int iRlen = gRight.size();
			unsigned int iLen = min(iLlen, iRlen);
			
			fout << "[" << ia << ", " << "size=" << iLen << "]" << "\t" << (*iter).GetWeight() << "\t";
			fout <<"\t";
			
			
			if(gLeft[gLeft.size()-1] == 1)
			{
			     for(vector<unsigned int>::iterator it = gLeft.end()-1; it != gLeft.begin(); --it)
                     fout << *it << " ";
				  fout << gLeft[0] << "," << endl;
			}
			else
			{
              for(vector<unsigned int>::iterator it = gRight.end()-1; it != gRight.begin(); --it)
                     fout << *it << " ";
			  fout << gRight[0] << "," << endl;			
			}
			
			++ia;
         }
		   		
		 fout << ";" << endl;			
		 fout << "END; [splits]" << endl;
     }
     
}

// generate full splits and output to file by Min method from alignment file
void CSplitSys::ProcessAlnMin(string& sInputFile, string& sOutputFile)
{
   // Read data from the weight file sInputFile
	
   cout << "-----------------------------------------------" << endl;
   cout << "Reading alignment file starts!" << endl; 
   
   ReadAlignment(sInputFile); 
   
   cout << "Reading alignment file finishes!" << endl; 
   cout << "-----------------------------------------------" << endl;
   cout << endl;

   cout << "-----------------------------------------------" << endl;
   cout << "Generating quintets starts!" << endl;
   
   GenerateQuintet();
      
   cout << "Generating quintets ends!" << endl;
   cout << "-----------------------------------------------" << endl;
   cout << endl;
   
   cout << "-----------------------------------------------" << endl;
   cout << "Calculating full non-trival split weights starts!" << endl;  
   ExpandAlignmentMin();
   
   cout << "Calculating full non-trivial splits endss!" << endl;
   cout << "-----------------------------------------------" << endl;
   cout << endl;
   
   cout << "-----------------------------------------------" << endl;
   cout << "Generating trivial splits starts!" << endl; 
   
   GenerateTrivialSplit();
   
   cout << "Generating trivial splits ends!" << endl;	 
   cout << "------------------------------------------------------" << endl;
	
   cout << "------------------------------------------------------" << endl;
   cout << "Writing the full splits to  the folder Output starts!" << endl;	
	
   Output(sOutputFile);
	
   cout << "Writing the full splits to  the folder Output ends!" << endl;	 
   cout << "------------------------------------------------------" << endl;
}                                      

// generate full splits and output to file by min method from weight file
void CSplitSys::ProcessWeiMin(string& sInputFile, string& sOutputFile)
{
    // Read data from the weight file sInputFile
	
   cout << "-----------------------------------------------" << endl;
   cout << "Reading alignment file starts!" << endl; 
   
   ReadWeight(sInputFile); 
   
   cout << "Reading alignment file finishes!" << endl; 
   cout << "-----------------------------------------------" << endl;
   cout << endl;

   cout << "-----------------------------------------------" << endl;
   cout << "Generating quintets starts!" << endl;
   
   GenerateQuintet();
      
   cout << "Generating quintets ends!" << endl;
   cout << "-----------------------------------------------" << endl;
   cout << endl;
   
   cout << "-----------------------------------------------" << endl;
   cout << "Calculating full non-trival split weights starts!" << endl;  
   ExpandAlignmentMin();
   
   cout << "Calculating full non-trivial splits endss!" << endl;
   cout << "-----------------------------------------------" << endl;
   cout << endl;
   
   cout << "-----------------------------------------------" << endl;
   cout << "Generating trivial splits starts!" << endl; 
   
   GenerateTrivialSplit();
   
   cout << "Generating trivial splits ends!" << endl;	 
   cout << "------------------------------------------------------" << endl;
	
   cout << "------------------------------------------------------" << endl;
   cout << "Writing the full splits to  the folder Output starts!" << endl;	
	
   Output(sOutputFile);
	
   cout << "Writing the full splits to  the folder Output ends!" << endl;	 
   cout << "------------------------------------------------------" << endl;
}      

// generate full splits and output to file by second method from alignment file
void CSplitSys::ProcessAlnSecond(string& sInputFile, string& sOutputFile)
{
    // Read data from the weight file sInputFile
	
   cout << "-----------------------------------------------" << endl;
   cout << "Reading alignment file starts!" << endl; 
   
   ReadAlignment(sInputFile); 
   
   cout << "Reading alignment file finishes!" << endl; 
   cout << "-----------------------------------------------" << endl;
   cout << endl;

   cout << "-----------------------------------------------" << endl;
   cout << "Generating quintets starts!" << endl;
   
   GenerateQuintetSecond();
      
   cout << "Generating quintets ends!" << endl;
   cout << "-----------------------------------------------" << endl;
   cout << endl;
   
   cout << "-----------------------------------------------" << endl;
   cout << "Calculating full non-trival split weights starts!" << endl;  
   ExpandAlignmentMin();
   
   cout << "Calculating full non-trivial splits endss!" << endl;
   cout << "-----------------------------------------------" << endl;
   cout << endl;
   
   cout << "-----------------------------------------------" << endl;
   cout << "Generating trivial splits starts!" << endl; 
   
   GenerateTrivialSplit();
   
   cout << "Generating trivial splits ends!" << endl;	 
   cout << "------------------------------------------------------" << endl;
	
   cout << "------------------------------------------------------" << endl;
   cout << "Writing the full splits to  the folder Output starts!" << endl;	
	
   Output(sOutputFile);
	
   cout << "Writing the full splits to  the folder Output ends!" << endl;	 
   cout << "------------------------------------------------------" << endl;
}  

// generate full splits and output to file by second method from weight file
void CSplitSys::ProcessWeiSecond(string& sInputFile, string& sOutputFile)
{
    // Read data from the weight file sInputFile
	
   cout << "-----------------------------------------------" << endl;
   cout << "Reading alignment file starts!" << endl; 
   
   ReadWeight(sInputFile); 
   
   cout << "Reading alignment file finishes!" << endl; 
   cout << "-----------------------------------------------" << endl;
   cout << endl;

   cout << "-----------------------------------------------" << endl;
   cout << "Generating quintets starts!" << endl;
   
   GenerateQuintetSecond();
      
   cout << "Generating quintets ends!" << endl;
   cout << "-----------------------------------------------" << endl;
   cout << endl;
   
   cout << "-----------------------------------------------" << endl;
   cout << "Calculating full non-trival split weights starts!" << endl;  
   ExpandAlignmentMin();
   
   cout << "Calculating full non-trivial splits endss!" << endl;
   cout << "-----------------------------------------------" << endl;
   cout << endl;
   
   cout << "-----------------------------------------------" << endl;
   cout << "Generating trivial splits starts!" << endl; 
   
   GenerateTrivialSplit();
   
   cout << "Generating trivial splits ends!" << endl;	 
   cout << "------------------------------------------------------" << endl;
	
   cout << "------------------------------------------------------" << endl;
   cout << "Writing the full splits to  the folder Output starts!" << endl;	
	
   Output(sOutputFile);
	
   cout << "Writing the full splits to  the folder Output ends!" << endl;	 
   cout << "------------------------------------------------------" << endl;
}                                      
                                    
             
// EOF


