/* File: CalWeiMin.cpp
 * Version 1.0 (AUG 25, 2010)
 * -----------------------------------------------------------------------------------------
 * Objective: calcualte weights of split types 2|n, 3|3 and m|n
 *
 *	Written by Jialiang Yang
 * -----------------------------------------------------------------------------------------
 */

#include "CSplitSys.h"
#include "auxilary.h"
#include "CQuintet.h"

// Calculate w(21|6543)
double CSplitSys::MinWei24(unsigned int gLeft[2], unsigned int gRight[4])
{  
   CQuintet tTemp;
  // 21|654
   unsigned int gRb[3];
   gRb[0] = gRight[0];
   gRb[1] = gRight[1];
   gRb[2] = gRight[2];
   tTemp.SetLabel(gLeft, gRb);
   double dMin = m_gQuintet[tTemp.GetIndex()].GetWeight();
   
   if(dMin <= 0) return 0;
   
   // 21|653
   gRb[2] = gRight[3];
   tTemp.SetLabel(gLeft, gRb);
   double dWei = m_gQuintet[tTemp.GetIndex()].GetWeight();
			 
   if(dWei <= 0) return 0;
   else if(dMin > dWei) dMin = dWei;
   
   // 21|643
   gRb[1] = gRight[2];
   tTemp.SetLabel(gLeft, gRb);
   dWei = m_gQuintet[tTemp.GetIndex()].GetWeight();
			 
   if(dWei <= 0) return 0;
   else if(dMin > dWei) dMin = dWei;
   
   // 21|543
   gRb[0] = gRight[1];
   tTemp.SetLabel(gLeft, gRb);
   dWei = m_gQuintet[tTemp.GetIndex()].GetWeight();
			 
   if(dWei <= 0) return 0;
   else if(dMin > dWei) dMin = dWei;
         
   return dMin;
}

// Calculate w(21|n...6543)
double CSplitSys::MinWei2n(const vector<unsigned int>& gLeft, const vector<unsigned int>& gRight)
{  
   double dMin = 10000000;
   double dWei;
   CQuintet tTemp;
   unsigned int gLb[2], gRb[3];
   gLb[0] = gLeft[0];
   gLb[1] = gLeft[1];
   
   vector<unsigned int>::const_iterator it1, it2, it3;

   for( it1 = gRight.begin(); it1 != gRight.end()-2; ++it1)
      for(it2 = it1+1; it2 != gRight.end()-1; ++it2)
	     for(it3 = it2+1; it3 != gRight.end(); ++it3)
		 {
			 gRb[0] = *it1;
			 gRb[1] = *it2;
			 gRb[2] = *it3;
			 tTemp.SetLabel(gLb, gRb);
			 dWei = m_gQuintet[tTemp.GetIndex()].GetWeight();
			 
             if(dWei <= 0) return 0;
             else if(dMin > dWei) dMin = dWei;
         }
   return dMin;
}


//  calculate w(123|456) 
double CSplitSys::MinWei33(unsigned int gLeft[3], unsigned int gRight[3])
{
   double dWei, dTemp;
   CQuintet tTemp;
   
   // case1: w(12|456)-w(12|3456)
   unsigned int gLb[2];
   gLb[0] = gLeft[0];
   gLb[1] = gLeft[1];
   unsigned int gRb[4];
   gRb[0] = gRight[0];
   gRb[1] = gRight[1];
   gRb[2] = gRight[2];
   gRb[3] = gLeft[2];
   
   tTemp.SetLabel(gLb, gRight);
   dTemp = m_gQuintet[tTemp.GetIndex()].GetWeight();
   dTemp -= MinWei24( gLb, gRb);
   dWei = dTemp;
   if( dWei <= 0) return 0;
   
   //case 2: w(13|456)-w(13|2456)
   swap(gLb[1], gRb[3]);
   tTemp.SetLabel(gLb, gRight);
   dTemp = m_gQuintet[tTemp.GetIndex()].GetWeight();
   dTemp -= MinWei24( gLb, gRb);
   if(dTemp <= 0) return 0;
   else if(dWei > dTemp) dWei = dTemp;
   
   //case 3: w(23|456)-w(23|1456)
   swap(gLb[0], gRb[3]);
   tTemp.SetLabel(gLb, gRight);
   dTemp = m_gQuintet[tTemp.GetIndex()].GetWeight();
   dTemp -= MinWei24( gLb, gRb);
   if(dTemp <= 0) return 0;
   else if(dWei > dTemp) dWei = dTemp;
        
   
   //case 4: w(45|123)-w(45|1236)
   unsigned int gLnew[2];
   gLnew[0] = gRight[0];
   gLnew[1] = gRight[1];
   unsigned int gRnew[4];
   gRnew[0] = gLeft[0];
   gRnew[1] = gLeft[1];
   gRnew[2] = gLeft[2];
   gRnew[3] = gRight[2];
   
   tTemp.SetLabel(gLnew, gLeft);
   dTemp = m_gQuintet[tTemp.GetIndex()].GetWeight();
   dTemp -= MinWei24( gLnew, gRnew);
   if(dTemp <= 0) return 0;
   else if(dWei > dTemp) dWei = dTemp;     
   
   //case 5: w(46|123)-w(46|1235)
   swap(gLnew[1], gRnew[3]);
   tTemp.SetLabel(gLnew, gLeft);
   dTemp = m_gQuintet[tTemp.GetIndex()].GetWeight();
   dTemp -= MinWei24( gLnew, gRnew);
   if(dTemp <= 0) return 0;
   else if(dWei > dTemp) dWei = dTemp; 
   
   //case 6: w(56|123)-w(56|1234)
   swap(gLnew[0], gRnew[3]);
   tTemp.SetLabel(gLnew, gLeft);
   dTemp = m_gQuintet[tTemp.GetIndex()].GetWeight();
   dTemp -= MinWei24( gLnew, gRnew);
   if(dTemp <= 0) return 0;
   else if(dWei > dTemp) dWei = dTemp; 
   
   return dWei;
}


//  calculate w(m|n)
double CSplitSys::MinWeimn(const vector<unsigned int>& gLeft, const vector<unsigned int>& gRight)
{
   double dWei = 10000000;
   double dTemp;
   unsigned int gLb[3], gRb[3];
   
   vector<unsigned int>::const_iterator it1, it2, it3, it4, it5, it6;
   
      for(it1 = gLeft.begin(); it1 != gLeft.end()-2; ++it1)
       for(it2 = it1 + 1; it2 != gLeft.end()-1; ++it2)
	     for(it3 = it2 +1; it3 != gLeft.end(); ++it3)
			for(it4 = gRight.begin(); it4 != gRight.end()-2; ++it4)
              for(it5 = it4 + 1; it5 != gRight.end()-1; ++it5)
	              for(it6 = it5+1; it6 != gRight.end(); ++it6)
		 {
			 gLb[0] = *it1;
			 gLb[1] = *it2;
			 gLb[2] = *it3;
			 gRb[0] = *it4;
			 gRb[1] = *it5;
			 gRb[2] = *it6;
			 
			 dTemp = MinWei33(gLb, gRb);  
			 if(dTemp <= 0) return 0;
			 else if(dWei > dTemp) dWei = dTemp;
         }
         
   return dWei;
}

// EOF
