/* File: ExpandAlignmentMin.cpp
 * Version 1.0 (AUG 25, 2010)
 * -----------------------------------------------------------------------------------------
 * Objective: expand the split system and generate full splits from quartets.
 *
 *	Written by Jialiang Yang
 * -----------------------------------------------------------------------------------------
 */
 
#include <string>
#include <iostream>
#include <fstream>
#include "CSplitSys.h"

using namespace std;

// Expand and generate the full splits from the fasta format file filename 
void CSplitSys::ExpandAlignmentMin()
{

   // Define sActive to store active splits and sTemp to store temp splits           
   vector<CSplit> gActive, gTemp;    
   vector<unsigned int> gLeft, gRight;
               
   // Case: k = 6 divides into two steps
   // Step 1: generating type 61|5432, 62|5431, 63|5421, 64|5321 65|4321 and their weights 
   unsigned int gLt[2], gRf[4];
   gLt[0] = 6;
   gLeft.push_back(6);
   gLt[1] = 1;
   gLeft.push_back(1);
   gRf[0] = 5;
   gRight.push_back(5);
   gRf[1] = 4;
   gRight.push_back(4);
   gRf[2] = 3;
   gRight.push_back(3);
   gRf[3] = 2;
   gRight.push_back(2);
   
   double dWei = MinWei24( gLt, gRf);
   CSplit sOne(gLeft, gRight, dWei);
   gActive.push_back(sOne);
    
   unsigned int i, k;   
   for(i = 1; i != 5; ++i)
   {
      swap(gLt[1], gRf[4-i]);    
      swap(gLeft[1], gRight[4-i]); 
      dWei = MinWei24( gLt, gRf);
	  
      CSplit sTemp(gLeft, gRight, dWei);
      gActive.push_back(sTemp);
    }
    
	// Step 2: generating type 21|6543, 621|543, 31|6542, 631|542 by adding 6 to the left 
	// and right side of 21|543 31|542, 41|532 ... and their weights.
	unsigned int gLL[3];
	vector<CQuintet>::iterator it23;
     for( it23 = m_gQuintet.begin(); it23 != m_gQuintet.begin()+10; ++it23)
   {
         vector<unsigned int> gLone, gRone;
         unsigned int *gL = (*it23).GetLblk();
         unsigned int *gR = (*it23).GetRblk();
         
		// add 6 to the left
		gLL[0] = 6;
		gLone.push_back(6);
		gLL[1] = gL[0];
		gLone.push_back(gL[0]);
		gLL[2] = gL[1];
		gLone.push_back(gL[1]);
		
		gRone.push_back(gR[0]);
		gRone.push_back(gR[1]);
		gRone.push_back(gR[2]);

		dWei = MinWei33(gLL, gR);
		CSplit sLeft(gLone, gRone, dWei);
		gActive.push_back(sLeft);
		
		// add 6 to the right 
		vector<unsigned int> gLtwo;
		gLtwo.push_back(gL[0]);
		gLtwo.push_back(gL[1]);
		
		gRf[0] = 6;
		gRf[1] = gR[0];
		gRf[2] = gR[1];
		gRf[3] = gR[2];
		gRone.insert(gRone.begin(), 6);
		
		dWei = MinWei24(gL, gRf);
		CSplit sRight(gLtwo, gRone, dWei);
		gActive.push_back(sRight);
	}
    	
    // delete the CSplits with weight 0
	ShrinkSplits(gActive); 
	
	vector<CSplit>::iterator iter;
    // Case: k >= 7
	for(k = 7; k != m_iNumber+1; ++k)
	{
        cout << "Step " << k << " Starts!" << endl;
        
		// As the same, there are two steps:
		// Step 1: generating type k1|(k-1)...2, k2|..., k3|..., k(k-1)|... and their weights 
		
		vector<unsigned int> gLb, gRb;
		gLb.push_back(k);
		gLb.push_back(1);
		
		for(i = k-1; i != 1; --i)
		     gRb.push_back(i);
   
        dWei = MinWei2n( gLb, gRb);
        CSplit sFirst(gLb, gRb, dWei);
        gTemp.push_back(sFirst);
      
        for(i = 1; i != k - 1; ++i)
       {
          swap(gLb[1], gRb[k-2-i]);    
          dWei = MinWei2n( gLb, gRb);
	  
          CSplit sTemp(gLb, gRb, dWei);
          gTemp.push_back(sTemp);
       }
		 
		 // Step 2: generating other types by adding k to the left and right hand side of active_CSplits 
		 
		 for( iter = gActive.begin(); iter != gActive.end(); ++iter)
    	{          
             // Add k to the left
	    	gLeft = (*iter).GetLblk();
	    	gRight = (*iter).GetRblk();
	    	gLeft.insert(gLeft.begin(), k);
		
            dWei = MinWeimn(gLeft, gRight);
                 
	     	CSplit sLeft(gLeft, gRight, dWei);
		    gTemp.push_back(sLeft);
		
		   // Add k to the right
	    	gLeft = (*iter).GetLblk();
	    	gRight = (*iter).GetRblk();
	    	gRight.insert(gRight.begin(), k);
		
            if(gLeft.size() == 2)
	             dWei = MinWei2n(gLeft, gRight);
            else
                 dWei = MinWeimn(gLeft, gRight);
                 
	     	CSplit sRight(gLeft, gRight, dWei);
		    gTemp.push_back(sRight);
	    }
	    
        // Delete the CSplits with weight 0
	    ShrinkSplits(gTemp);
	    gActive = gTemp;
	    gTemp.clear();
	    
	    cout<<"Step "<<k<<" Ends!"<<endl;
    }
	
	m_gSplit = gActive;
}
