/* File: auxilary.h
 * Version 1.0 (AUG 24, 2010)
 * --------------------------------------------------------------------------
 * Objective: Declare some auxilary functions that will be used
 *
 **        Binom: Return the binomial number. e.g Binom(n,k) returns n chooses k
 **   CalTriplet: Calculate the number of sites in string s1, s2 and s3 which supports s1|s2s3
 **   CalQuartet: Calculate the number of sites in string s1, s2, s3 and s4 which supports s1s2|s3s4
 **         trim: Remove spaces in the begining and end of a string
 **      DiffVec: Check if all the elements in T are different
 **      SameVec: Check if all the elements in T are the same
 *
 *  Written by Jialiang Yang
 * --------------------------------------------------------------------------
 */
 
#ifndef _AUXILARY_H
#define _AUXILARY_H

//Body of auxilary.h
#include <vector>
#include <string>

using namespace std;
                    
extern unsigned long long Binom(unsigned int, unsigned int);                                 // Return the number: n chooses m
extern string trim(string&);                                                                 // Remove spaces in the begining and end of a string
extern unsigned int CalTriplet(const string&, const string&, const string&);                 // Calculate the weights supporting s1|s2s3
extern unsigned int CalQuartet(const string&, const string&, const string&, const string&);  // Calculate the weights supporting s1s2|s3s4

// Check if all the elements in T are different
template <typename T> bool DiffVec(const vector<T>& gVec)
{
    typename vector<T>::const_iterator it1, it2;
    
    for(it1 = gVec.begin(); it1 != gVec.end()-1; ++it1)
        for(it2 = it1+1; it2 != gVec.end(); ++it2)
       {
            if(*it1 == *it2) return false;
        }
        
   return true;   
}

// Check if all the elements in T are the same
template <typename T> bool SameVec(const vector<T>& gVec)
{
    typename vector<T>::const_iterator it1, it2;
    
    for(it1 = gVec.begin(); it1 != gVec.end()-1; ++it1)
        for(it2 = it1+1; it2 != gVec.end(); ++it2)
       {
            if(*it1 != *it2) return false;
        }
        
   return true;   
}

#endif
//EOF
