#include <iostream>
#include <time.h>

#include "CSplitSys.h"

using namespace std;

const double dVERSION = 1.0;

int main(unsigned int argc, char *argv[])
{
    unsigned int g_Alignment = 1;
	unsigned int g_Min = 1; 
	 
   // Read options from command line:
	for(unsigned int n=1; n < argc; n++)
	{
		if(argv[n][0]=='-')
		{	
			switch(argv[n][1])
			{
				case '-': // Show help:
					cout << "Usage: TtailyNet [-params] " << endl;
					cout << "TailyNet Version: " << dVERSION << endl;
					cout << "Options: -- Show usage " << endl;
				    cout << "         -a Read data from alignment files (Default)." << endl;
				    cout << "         -w Read data from weight files." << endl;
					cout << "         -m Calculate quintet weights by Minimum method (Default)." << endl;
					cout << "         -s Calculate quintet weights by Second Minimum method." << endl;
					//cout << "         -v Check the validality of the input" << endl;
					exit(0);
					
			   case 'a': // Read data from multiple alignment file with fasta format
                    g_Alignment = 1;
					break;
				
			   case 'w': // Read data from weight files
                    g_Alignment = 0;
					break;
				
				case 'm': // Calculate quintet weights by Minimum method
                    g_Min = 1;
					break;
					
				case 's': // Calculate quintet weights by Second Minimum method
                    g_Min = 0;
					break;	
					
				/*case 'v': // Check the validality of the Newick tree, it slows down the program
                    g_Valid = 0;
					break;*/
					
   				default: // No such option:
					cout << "Illegal option: " << argv[n] << " ! " << endl;
					exit(0);
			}
		}
		else
			break; // this must be the first file name
	}
	
	
    // Codes start here

   string sInputFile, sOutputFile;
   
   if(g_Alignment == 1)
   {
      cout << "Please input the path for the multiple alignment file in fasta format: " << endl;
      cin >> sInputFile;
   }
   else 
   {
      cout << "Please input the path for the valid weight file: " << endl;
      cin >> sInputFile;
   }
	   
   cout << "Please specify a name for output, by default it will have a suffix .nex: " << endl;
   cin >> sOutputFile;
	
   // Record running time
    clock_t cStart, cFinish;
    double dTotalTime;
    cStart = clock();
	
	CSplitSys aSys;
	
	if (g_Alignment == 1)
	{
	    if(g_Min == 1) aSys.ProcessAlnMin(sInputFile, sOutputFile);
		else  
		     aSys.ProcessAlnSecond(sInputFile, sOutputFile);
	}
	else
	{
	   if(g_Min == 1) aSys.ProcessWeiMin(sInputFile, sOutputFile);
		else  
		     aSys.ProcessWeiSecond(sInputFile, sOutputFile);
	}
    
    cFinish  = clock();
    
    dTotalTime = (double)(cFinish-cStart)/CLOCKS_PER_SEC;
    
    cout << "Running Time: " << dTotalTime << " Seconds." << endl;
    
    //system("Pause");
    return 0;
}
